expit = function(X,beta){
    Xbeta = X %*% beta
    out = 1/(1+exp(-Xbeta))
    return(out)
}
gam.init.sub = function(q,n.initials){   
          out=rmvnorm(n.initials,mean=rep(0,q),sigma=diag(rep(1,q)))
          dd=apply(out^2,1,sum)
          out=out/sqrt(dd)
    return(out)}
    
#####################################################################################
## gam.init #########################################################################
## Function to select a few gamma value spanning its space
## Input: 
## n.initials: number of gamma values
## q: dimension of covariate for defining subgroup
## lb.quantile, ub.quantile: lower and upper quantile specified for Z%*%gamma.initials
## ss: a positive integer with n.initials/ss indicating how many sets of 
## {gamma_1,...,gamma_q} are chosen, default to 1 
## 
## Output: 
## A set of gamma values selected for defining subgroup
######################################################################################
        
  gam.init = function(n.initials,q,Z,lb.quantile,ub.quantile,ss=1){

    if(q==1){
        gamma.initials = matrix(1,n.initials,q+1)
        gamma.initials[,1] = -quantile(Z,seq(lb.quantile,ub.quantile,length=n.initials))
    }else{
        gamma.initials = gam.init.sub(q,n.initials/ss)
        Z.gamma.initials = Z %*% t(gamma.initials)
        ll=round(n.initials/ss)
        qtile = sample(seq(lb.quantile,ub.quantile,length=n.initials),n.initials)
        gamma.initials.1 = sapply(1:n.initials,function(x)return(
                            -quantile(Z.gamma.initials[,x-floor((x-0.1)/ll)*ll],qtile[x])
                        ))
                        
        gamma.initials.1=ifelse(gamma.initials.1==(-1)*apply(Z.gamma.initials,2,min),gamma.initials.1-0.001,gamma.initials.1)
        gamma.initials.1=ifelse(gamma.initials.1==(-1)*apply(Z.gamma.initials,2,max),gamma.initials.1+0.001,gamma.initials.1)
        gamma.initials.aug=do.call("rbind", rep(list(gamma.initials), ss))
        gamma.initials = cbind(gamma.initials.1,gamma.initials.aug)
    }
    return(gamma.initials)
    }
      
######################################################################################################
## chngpt.mvtest.2phase.fast.full ####################################
## Function to test for existence of subgroup allowing for two-phase sampling

## Model: logit P(y=1|X,Z)=beta1^T(1,X)+beta2^TXs I(gamma^T(1,Z)>0), and we
## test for H_0: beta2=0

## Input:

## y: binary disease outcome (taking value 0,1)
## X: covariates for modeling disease risk model
## oo: indicator of a subset of (1,X): Xs=(1,X)[oo] where it is of interest to 
## test whether the relationship between y and Xs differ according to subgroups 
## defined by Z
## Z: covariates (biomarkers) for defining subgroup with differential risk models
## prob.weight: sampling probability for phase-2 sample in two-phase sampling
##     for iid sample, just input prob.weight=rep(1,length(y))
## n.initials,lb.quantile, ub.quantile, ss are input for gam.init function for selecting
##          a set of gamma values
## n.boots: number of resamples for p-value computation
## C: indicator for covariate strata for two-phase sampling, assuming positive sampling
##    probability within each sampling stratum defined by (y,C)
##    for iid sample, just enter C=rep(1,length(y))

## Output: 

## LR: maximum of Weighted LR (WLR) test statistics 
## pval: p-value based on LR  
## LR.hat: maximum of WLR test statistics computed using the formula presented  in 
##      section 2.2 of Huang, Cho, Fong 2020
## pval.hat: p-value based on LR.hat (recommended for better control of type-I error
##       in scenarios of multivariate X compared to pval)  
######################################################################################
           
    
chngpt.mvtest.2phase.fast.full = function(y,X,Z,oo,prob.weight,
                n.initials=NULL, 
                lb.quantile=.1, ub.quantile=.9,
                n.boots=2000,C=NULL,
                ss=1){


    s=length(oo)
    
    n = length(y)
    ifelse(is.matrix(X),p<-ncol(X),p<-1) ## dimension of X
    ifelse(is.matrix(Z),q<-ncol(Z),q<-1) ## dimension of Z
    X1 = cbind(1,X)
    Z1 = cbind(1,Z)
    
    #######################
    ## generate initials ##
    #######################
    
    if(is.null(n.initials)) n.initials = 100
    gamma.initials = gam.init(n.initials,q,Z,lb.quantile=.1,ub.quantile,ss)
    ind.initials = Z1 %*% t(gamma.initials)>0
    
    ##############
    ## under.H0 ##
    ##############
    
    glm0 = glm(y~X,family=binomial(link = "logit"),weights=1/prob.weight)
    beta1hat = glm0$coef
    loglike.null = glm0$dev # equals to -log(Lhat)
    mu.null = expit(cbind(1,X), beta1hat)
    D.null.p = Diagonal(n,x=c(mu.null*(1-mu.null))/prob.weight)
    y_mu.null = y - mu.null
    
    
    ##########################
    ## compute loglikeRatio ##
    ##########################
    
    LRcandid = LRcandid.hat=numeric(n.initials)
    LRcandid0 = LRcandid0.hat=numeric(n.initials)
    
    I.b1 = t(X1) %*% D.null.p %*% X1
    I.b1.inv = solve(I.b1)
    G = rbind(diag(1,p+1),matrix(0,s,p+1))  
    
    
    G.I.b1.inv.G = G %*% I.b1.inv %*% t(G)
    
    
    In.inv = sparseMatrix(i=1,j=1,x=0,dims=c(n.initials*(1+p+s),n.initials*(1+p+s)))  ### modified 04/20
    
    ones2 = Matrix(0,n.initials,(p+1+s)*n.initials)
    
    
    XW = c()
    which.err = c()
    iter = 0
    for(j in 1:n.initials){
    
        ind = ind.initials[,j]
        
        Wj = X1[,oo] * as.numeric(ind)  
        
        XWj = cbind(X1,Wj)
        XW = cbind(XW,XWj)
    
        
        glm1 = glm(y~cbind(X,Wj),family=binomial(link = "logit"),weights=1/prob.weight)
        beta1hatj = glm1$coef
        loglike.alt = glm1$dev
        LRcandid[j] = loglike.null - loglike.alt
    
    
        I.b = (t(XWj) %*% D.null.p) %*% XWj
        I.b.inv = solve(I.b)
        In.inv[(j-1)*(p+1+s)+1:((p+1+s)),(j-1)*(p+1+s)+1:((p+1+s))] = I.b.inv - G.I.b1.inv.G   
        
    
        Snj = t(XWj) %*% (y_mu.null/prob.weight)
        LRcandid.hat[j]=t(Snj)%*%(I.b.inv - G.I.b1.inv.G)%*%Snj
    
        ones2[j,(j-1)*(p+1+s)+1:((p+1+s))] = 1   
    }
    LR = max(LRcandid)
    LR.hat = max(LRcandid.hat)
    
    ###############################
    ## compute empirical p-value ##
    ###############################
    
    std.norm = matrix(rnorm(n*n.boots),n.boots)
    
    D0 = (mu.null^2)*(1-mu.null)
    D1 = mu.null*(1-mu.null)^2
    
    
    prob.weight.0<-prob.weight[y==0]
    prob.weight.1<-prob.weight[y==1]
    C.0<-C[y==0]
    C.1<-C[y==1]
    Cu<-sort(unique(C))
    p.0<-prob.weight.0[match(Cu,C.0)]
    p.1<-prob.weight.1[match(Cu,C.1)]
    
    for(i in 1:length(Cu)){#i=1
        here = which(C == Cu[i])
        D0[here] = D0[here]/p.0[i]
        D1[here] = D1[here]/p.1[i]
    }
    
    
    D.null.half.p = Diagonal(n,x=sqrt(c(D0+D1)/prob.weight))
    
    bootsamp7 = std.norm %*% (D.null.half.p %*% XW)
    hey = bootsamp7 %*% In.inv
    heyhey = hey*bootsamp7
    bs.out7 = apply(heyhey,1,function(x)return(max(ones2 %*% x)))
    pval = mean(bs.out7>LR)
    pval.hat=mean(bs.out7>LR.hat)
    
    return(list(LR=LR,pval=pval,LR.hat=LR.hat,pval.hat=pval.hat))

}
