library(mvtnorm)
library(MASS)
library(Matrix)
library(kyotil)
library(glmnet)

load(file="exampleData.Rdata")  

source(file="FunctionOneStep.R")
source(file="FunctionTwoStep.R")


#### One-step testing for a two-phase sample

set.seed(1) 

out11.ts = chngpt.mvtest.2phase.fast.full(dat.ts$y,dat.ts$X,dat.ts$Z,oo=(1:(ncol(as.matrix(dat.ts$X))+1)),prob.weight=rep(1,length(dat.ts$y)),n.initials=1000, 
                lb.quantile=.1, ub.quantile=.9, 
                n.boots=1000,C=dat.ts$C,ss=1)

#> out11.ts
#$LR
#[1] 23.40453
#
#$pval
#[1] 0.334
#
#$LR.hat
#[1] 24.4024
#
#$pval.hat
#[1] 0.246


#### One-step testing for an iid sample, set "prob.weight" to a vector of 1's and set "C" to a vector of equal value

set.seed(1)
out11.iid = chngpt.mvtest.2phase.fast.full(dat.iid$y,dat.iid$X,dat.iid$Z,oo=(1:(ncol(as.matrix(dat.iid$X))+1)),prob.weight=rep(1,length(dat.iid$y)),n.initials=1000, 
                lb.quantile=.1, ub.quantile=.9,
                n.boots=1000,C=rep(1,length(dat.iid$y)),ss=1)
#>out11.iid
#$LR
#[1] 11.41237
#
#$pval
#[1] 0.747
#
#$LR.hat
#[1] 10.9155
#
#$pval.hat
#[1] 0.8
               
                
#### Two-step testing for an iid sample                

### compute test statistics when univariate combination for subgroup score is derived based on main effect of Z only


teststat.XZ= compute.teststat.full(dat.iid$X,dat.iid$Z,dat.iid$y,oo=(1:(ncol(as.matrix(dat.iid$X))+1)),interaction='no',n.initials=50,sel=F)

### compute test statistics when univariate combination for subgroup score is derived based on interaction between X and Z

teststat.XZXZ= compute.teststat.full(dat.iid$X,dat.iid$Z,dat.iid$y,oo=(1:(ncol(as.matrix(dat.iid$X))+1)),interaction='yes',n.initials=50,sel=F)

#################################
## find reference distribution ##
nBoots=100
glm.X = glm(dat.iid$y ~ dat.iid$X, family=binomial(link='logit'))
Xb1.X = cbind(1,dat.iid$X) %*% glm.X$coef
mu.X = 1/(1+exp(-Xb1.X))
n=length(dat.iid$y)
y.X = matrix(rbinom(n*nBoots,1,mu.X),n,nBoots,byrow=FALSE)

set.seed(1) 
bootsamp.X = matrix(0,nBoots,2)
for(i in 1:nBoots){#i=1
    y.X.i = y.X[,i]
    set.seed(i)
    teststat.XZXZ.b1 = compute.teststat.full(dat.iid$X,dat.iid$Z,y.X.i,oo=(1:(ncol(as.matrix(dat.iid$X))+1)),interaction='yes',
    n.initials=50,sel=F)
    teststat.XZ.b1 = compute.teststat.full(dat.iid$X,dat.iid$Z,y.X.i,oo=(1:(ncol(as.matrix(dat.iid$X))+1)),interaction='no',
    n.initials=50,sel=F)
    bootsamp.X[i,] = c(teststat.XZXZ.b1,teststat.XZXZ.b1)
    if(i%%100==0)cat(".")
}

#########################################
## compute p-value for two-step testing##
pval.XZ = mean(teststat.XZ<bootsamp.X[,1])
pval.XZXZ = mean(teststat.XZXZ<bootsamp.X[,2])
                
#> pval.XZ
#[1] 0.7
#> pval.XZXZ
#[1] 0.73
               
